<?php

class Backup_File_Converter {

	function convert( $file )
	{
		$fp_old = fopen( $file, 'r' );

		if ( ! $fp_old )
		{
			return FALSE;
		}

		$new_file = dirname( $file ) . '/' . 'backup_converted.php';
		$fp_new = fopen( $new_file, 'w' );

		if ( ! $fp_new )
		{
			return FALSE;
		}

		if ( substr( $file, -4 ) == '.php' )
		{
			fgets( $fp_old );
		}

		fwrite( $fp_new, "<?php die('No direct access allowed'); ?>\n");
		$table = '';
		$schema = '';
		$dataTables = FALSE;

		fwrite( $fp_new, "#version: 1.0\n" );
		fwrite( $fp_new, '#date: ' . date( 'Y-m-d h:i' ) . "\n" );

		while ( $line = $this->_read_line_from_backup_file( $fp_old ) )
		{
			if ( substr( $line, 0, 22 ) == '---begin of dataTables' )
			{
				$dataTables = TRUE;
			}
			elseif ( substr( $line, 0, 20 ) == '---end of dataTables' )
			{
				$dataTables = FALSE;
			}
			elseif ( substr( $line, 0, 12 ) == '---begin of ' )
			{
				$table = ( $dataTables ? 'data_' : '' ) . substr( $line, 12, -3 );
				$fields = '';
				$schema = '';
			}
			elseif ( ! in_array( $table, array( '', 'index', 'session', 'tree' ) ) 
						&& substr( $line, 0, 2 ) == 'a:' )
			{
				$record = unserialize( $line );
				$new_record = array();

				if ( ! $record )
				{
					echo "Could not unserialize $line \n";
					continue;
				}

				if ( empty( $fields ) )
				{
					$fields = array_keys( $record );
					$schema = $this->_get_schema( $table );

					if ( $dataTables )
					{
						foreach ( $fields as $field )
						{
							if ( in_array( $field, array( 'vtid', 'vtlanguage' ) ) )
							{
								continue;
							}

							$type = in_array( $field, array( 'vtmedia', 'vtfiles' ) ) ? 'longtext' : 'text';
							$schema[] = "`{$field}` {$type}";
						}

						$schema[] = "PRIMARY KEY (`vtid`,`vtlanguage`)";
					}

					fwrite( $fp_new, "\n" );
					fwrite( $fp_new, "#table: {$table}\n" );
					fwrite( $fp_new, '#schema:' . implode( ', ', $schema ) . "\n" );
					fwrite( $fp_new, '#fields: `' . implode( '`, `', $fields ) . "`\n" );
				}

				foreach( $record as $field )
				{
					$new_record[] = addcslashes( $field, "\x00\n\r\\'\"\x1a" );
				}

				fwrite( $fp_new, "'" . implode( "','", $new_record ) . "'\n" );
			}
		}

		fclose( $fp_old );
		fclose( $fp_new );
		return $new_file;
	}
	
	function _get_schema( $table )
	{
		$table = substr( $table, 0, 5 ) == 'data_' ? 'data' : $table;

		switch ( $table )
		{
			case 'content':
				$fields[] = "`id` int(11) NOT NULL auto_increment";
				$fields[] = "`containerID` int(11) default '0'";
				$fields[] = "`package` text";
				$fields[] = "`type` text";
				$fields[] = "`creationTime` int(14) default '0'";
				$fields[] = "`name` text";
				$fields[] = "`modificationTime` int(14) default '0'";
				$fields[] = "`path` text";
				$fields[] = "`inTrash` tinyint(4) unsigned default '0'";
				$fields[] = "`sortField` text";
				$fields[] = "`sortOrder` text";
				$fields[] = "`sortValue` int(11) default '0'";
				$fields[] = "`comment` text";
				$fields[] = "`userID` int(11) default '0'";
				$fields[] = "`ownerID` int(11) unsigned default '0'";
				$fields[] = "`accessRights` smallint(6) default '0'";
				$fields[] = "`aliasID` int(11) default '0'";
				$fields[] = "`itemCount` int(10) unsigned default '0'";
				$fields[] = "`creationUser` text";
				$fields[] = "`modificationUser` text";
				$fields[] = "`inheritRights` tinyint(4) unsigned default '0'";
				$fields[] = "`invisible` tinyint(3) unsigned default '0'";
				$fields[] = "`keywords` text";
				$fields[] = "`label` int(10) unsigned default '0'";
				$fields[] = "`icon` text";
				$fields[] = "`description` text";
				$fields[] = "`titel` text";
				$fields[] = "`dataStore` longtext";
				$fields[] = "`internalHits` int(10) unsigned default '0'";
				$fields[] = "`externalHits` int(11) default '0'";
				$fields[] = "PRIMARY KEY (`id`)";
				return $fields;

			case 'data':
				$fields[] = "`vtid` int(11) unsigned NOT NULL default '0'";
				$fields[] = "`vtlanguage` char(3) NOT NULL DEFAULT ''";
				return $fields;

			case 'session':
				$fields[] = "`id` int(11) NOT NULL auto_increment";
				$fields[] = "`accessTime` int(14) default '0'";
				$fields[] = "`code` text";
				$fields[] = "`IPaddress` text";
				$fields[] = "`originPageID` int(11) default '0'";
				$fields[] = "`originMethod` text";
				$fields[] = "`loginTime` int(14) default '0'";
				$fields[] = "`updateNavigator` tinyint(4) default '0'";
				$fields[] = "`expandedPages` text";
				$fields[] = "`markedPages` text";
				$fields[] = "`message` longtext";
				$fields[] = "`language` char(3) default NULL";
				$fields[] = "`userID` int(11) default '0'";
				$fields[] = "`loggedIn` tinyint(4) default '0'";
				$fields[] = "`logoutTime` int(11) default '0'";
				$fields[] = "`dataStore` longtext";
				$fields[] = "`selections` longtext";
				$fields[] = "PRIMARY KEY (`id`)";
				return $fields;

			case 'tree':
				$fields[] = "`id` int(11) unsigned NOT NULL default '0'";
				$fields[] = "`pos` bigint(20) unsigned NOT NULL default '0'";
				$fields[] = "`forkneedsupdate` tinyint(1) unsigned NOT NULL default '0'";
				$fields[] = "PRIMARY KEY  (`id`)";
				return $fields;

			case 'user':
				$fields[] = "`id` int(11) NOT NULL default '0'";
				$fields[] = "`name` varchar(120) binary NOT NULL default ''";
				$fields[] = "`password` varchar(40) NOT NULL default '0'";
				$fields[] = "`is_group` tinyint(4) default '0'";
				$fields[] = "`admin` tinyint(4) default '0'";
				$fields[] = "`startPage` varchar(40) default ''";
				$fields[] = "`workspace` tinyint(4) default '0'";
				$fields[] = "`active` tinyint(4) default '0'";
				$fields[] = "`realname` text";
				$fields[] = "PRIMARY KEY (`id`)";
				$fields[] = "UNIQUE KEY `name` (`name`)";
				return $fields;

			default:
				return array();
		}
	}


	function _read_line_from_backup_file( $fp )
	{
		if ( ! $line = fgets( $fp ) )
		{
			return FALSE;
		}

		return stripcslashes( substr( $line, 0, -1 ) );
	}


}

// end of file