<?php

class Image_Processor {
	
	var $settings;
	var $image_file;
	var $image;
	var $metadata_irb = array();
	var $metadata_xmp = array();
	
	
	function Image_Processor( $settings = array() )
	{
		$this->settings = array(
			'watermarkMargin' => 15,
			'fontPath' => dirname( __FILE__ ) . '/../resources/bitstream-vera/'
		);
		
		if ( ! empty( $settings ) )
		{
			$this->settings = array_merge( $this->settings, $settings );
		}
	}


	function load( $image_file, $preserve_metadata = FALSE )
	{
		list( $width, $height ) = getimagesize( $image_file );
		$memory_need = $width * $height * 3;
		
		if ( $memory_need > $this->_get_available_memory() / 3 )
		{
			ini_set( 'memory_limit', $memory_need );
			
			if ( ini_get( 'memory_limit' ) != $memory_need )
			{
				return FALSE;
			}
		}

		$this->image_file = $image_file;
		$this->image = imagecreatefromjpeg( $image_file );

		if ( $preserve_metadata )
		{
			$meta = new Metadata_Manager( $image_file );
			$this->metadata_irb = $meta->get_irb_data();
			$this->metadata_xmp = $meta->get_xmp_data();
		}

		return TRUE;
	}


	function resize( $size )
	{
		// TODO: Minimale und maximale Seitenlänge als Vorgabe (z. B. für Quadrate und Panoramafotos)

		$width = imagesx( $this->image );
		$height = imagesy( $this->image );
		$image_ratio = $width / $height;

		$max_width = $size;
		$max_height = $size;
		$new_width = $max_width;
		$new_height = $max_height;

		if ( $width > $max_width AND $height < $width )
		{
	  		$new_height = ceil( $max_width / $image_ratio );
		}
		elseif( $height > $max_height AND $height > $width )
		{
	  		$new_width = ceil( $max_height * $image_ratio );
		}
		elseif ( $height < $max_height AND $height > $width )
		{
	  		$new_width = ceil( $max_height * $image_ratio );
		}
		elseif ( $width < $max_width AND $height < $width )
		{
	  		$new_height = ceil( $max_width * $image_ratio );
		}
		elseif ( $height == $width )
		{
			$new_height = $new_width;
		}

		$image_new = imagecreatetruecolor( $new_width, $new_height );
		imagecopyresampled( $image_new, $this->image, 0, 0, 0, 0, $new_width, $new_height, $width, $height );
		imagedestroy( $this->image );
		$this->image = $image_new;
	}


	function add_watermark_text( $text, $alignment = 'c', $opacity = 70, $size = 0.7, $font = 'Vera.ttf', $color = 'ffffff', $rotation = 0 )
	{
		// This function is based on source code from
		// http://salman-w.blogspot.com/2008/11/watermark-your-images-with-text-using.html

		// TODO: Margin automatisch anhand der Schrift-/Bildgröße setzen


		$image_width = imagesx( $this->image );
		$image_height = imagesy( $this->image );
		
		if ( $size <= 0 )
		{
			return FALSE; // invalid size
		}
		elseif ( $size <= 1 )
		{
			// Interprete the size as a scaling factor for the text box
			// relative to the image width

			$box = $this->_imagettfbbox_fixed( 24, $rotation, $font, $text );
			$size = 24 * $size * $image_width / ( $box[2] - $box[1] );
		}

		
		$box = $this->_imagettfbbox_fixed( $size, $rotation, $font, $text );
		$x0 = min( $box[0], $box[2], $box[4], $box[6] ) - $this->settings['watermarkMargin'];
		$x1 = max( $box[0], $box[2], $box[4], $box[6] ) + $this->settings['watermarkMargin'];
		$y0 = min( $box[1], $box[3], $box[5], $box[7] ) - $this->settings['watermarkMargin'];
		$y1 = max( $box[1], $box[3], $box[5], $box[7] ) + $this->settings['watermarkMargin'];
		$box_width = abs( $x1 - $x0 );
		$box_height = abs( $y1 - $y0 );

		switch ( $alignment )
		{
			case 'nw':
				$bpy = -$y0;
				$bpx = -$x0;
				break;
			case 'n':
				$bpy = -$y0;
				$bpx = $image_width / 2 - $box_width / 2 - $x0;
				break;
			case 'ne':
				$bpy = -$y0;
				$bpx = $image_width - $x1;
				break;
			case 'w':
				$bpy = $image_height / 2 - $box_height / 2 - $y0;
				$bpx = -$x0;
				break;
			case 'c':
				$bpy = $image_height / 2 - $box_height / 2 - $y0;
				$bpx = $image_width / 2 - $box_width / 2 - $x0;
				break;
			case 'e':
				$bpy = $image_height / 2 - $box_height / 2 - $y0;
				$bpx = $image_width - $x1;
				break;
			case 'sw':
				$bpy = $image_height - $y1;
				$bpx = -$x0;
				break;
			case 's':
				$bpy = $image_height - $y1;
				$bpx = $image_width / 2 - $box_width / 2 - $x0;
				break;
			case 'se';
				$bpy = $image_height - $y1;
				$bpx = $image_width - $x1;
				break;
		}

		$alpha_color = imagecolorallocatealpha(
			$this->image,
			hexdec( substr( $color, 0, 2 ) ),
			hexdec( substr( $color, 2, 2 ) ),
			hexdec( substr( $color, 4, 2 ) ),
			127 * ( 100 - $opacity ) / 100
		);

		$shadow_color = imagecolorallocatealpha( $this->image, 0, 0, 0, 127 * ( 100 - $opacity ) / 100 );

		imagettftext( $this->image, $size, $rotation, $bpx + 2, $bpy + 2, $shadow_color, $this->settings['fontPath'] . $font, $text );
		imagettftext( $this->image, $size, $rotation, $bpx, $bpy, $alpha_color, $this->settings['fontPath'] . $font, $text );
	}


	function add_watermark_image( $watermark_file, $alignment = 'c', $opacity = 70, $scaling = 0.7 )
	{
		// This function is based on source code from
		// http://salman-w.blogspot.com/2008/11/watermark-your-images-with-another.html

		// Watermark Image should be created with the following settings:
		// File Format: PNG-8, non-interlaced
		// Color Reduction: Selective, 256 colors
		// Dithering: Diffusion, 88%
		// Transparency: On, Matte: None
		// Transparency Dither: Diffusion Transparency Dither, 100%

		$image_width = imagesx( $this->image );
		$image_height = imagesy( $this->image );
		echo $scaling;
		$scaling = max( 0, min( 1, $scaling ) );
		echo $scaling;

		$watermark = imagecreatefrompng( $watermark_file );
		$watermark_width = imagesx( $watermark );
		$watermark_height = imagesy( $watermark );

		// Resize watermark if neccessary
		// If the $size parameter is zero, the watermark is placed in its original size
		// Otherwise it is scaled by $scaling (in relation to the narrow side of the original image)
		
		if ( $scaling )
		{
			$watermark_ratio = $watermark_width / $watermark_height;

			if ( $image_width < $image_height )
			{
				$watermark_width = $image_width * $scaling;
				$watermark_height = $watermark_width / $watermark_ratio;
			}
			else
			{
				$watermark_height = $image_height * $scaling;
				$watermark_width = $watermark_height * $watermark_ratio;
			}

			$new_watermark = imagecreatetruecolor( $watermark_width, $watermark_height );
			imagealphablending( $new_watermark, FALSE );
			imagesavealpha( $new_watermark, TRUE );
			$transparent = imagecolortransparent( $new_watermark, imagecolorallocatealpha( $new_watermark, 0, 0, 0, 127 ) );
			// $transparent = imagecolortransparent( $new_watermark, imagecolorallocatealpha( $new_watermark, 0, 0, 0, 127 ) );
			imagefill( $new_watermark, 0, 0, $transparent );
			imagecopyresized( $new_watermark, $watermark, 0, 0, 0, 0, $watermark_width, $watermark_height, imagesx( $watermark ), imagesy( $watermark ) );
			imagedestroy( $watermark );
			$watermark = $new_watermark;
		}
		
		switch ( $alignment )
		{
			case 'nw':
				$x = 0;
				$y = 0;
				break;
			case 'n':
				$x = $image_width / 2 - $watermark_width / 2;
				$y = 0;
				break;
			case 'ne':
				$x = $image_width - $watermark_width;
				$y = 0;
				break;
			case 'w':
				$x = 0;
				$y = $image_height / 2 - $watermark_height / 2;
				break;
			case 'c':
				$x = $image_width / 2 - $watermark_width / 2;
				$y = $image_height / 2 - $watermark_height / 2;
				break;
			case 'e':
				$x = $image_width - $watermark_width;
				$y = $image_height / 2 - $watermark_height / 2;
				break;
			case 'sw':
				$x = 0;
				$y = $image_height - $watermark_height;
				break;
			case 's':
				$x = $image_width / 2 - $watermark_width / 2;
				$y = $image_height - $watermark_height;
				break;
			case 'se':
				$x = $image_width - $watermark_width;
				$y = $image_height - $watermark_height;
				break;
			default:
				$x = $image_width / 2 - $watermark_width / 2;
				$y = $image_height / 2 - $watermark_height / 2;
		}
		
		imagecopymerge( $this->image, $watermark, $x, $y, 0, 0, $watermark_width, $watermark_height, $opacity );
		imagedestroy( $watermark );
	}


	function write( $file, $quality = 90 )
	{
		$parts = explode( '/', dirname( $file ) );
		$dir = '';

		foreach ( $parts as $part )
		{
			$dir .= $part . '/';

			if ( ! is_dir( $dir ) && strlen( $dir ) > 0 )
			{
				mkdir( $dir );
				chmod( $dir, 0777 );
			}
		}

		imagejpeg( $this->image, $file, $quality );

		if ( ! empty( $this->metadata_irb ) || ! empty( $this->metadata_irb ) )
		{
			$meta = new Metadata_Manager( $file );
			$meta->put_irb_data( $this->metadata_irb );
			$meta->put_xmp_data( $this->metadata_xmp );
			$meta->write( $file );
		}
	}


	function _imagettfbbox_fixed( $size, $rotation, $font, $text )
	{
		 // Fix for the buggy imagettfbbox implementation in gd library

		$box = imagettfbbox( $size, 0, $this->settings['fontPath'] . $font, $text );
		$angle = deg2rad( $rotation );
		$result = array();

		for ( $i = 0; $i < 7; $i += 2 )
		{
			$result[ $i + 0 ] = round( $box[ $i + 0 ] * cos( $angle ) + $box[ $i + 1 ] * sin( $angle ) );
			$result[ $i + 1 ] = round( $box[ $i + 1 ] * cos( $angle ) - $box[ $i + 0 ] * sin( $angle ) );
		}

		return $result;
	}


	function _get_available_memory()
	{
		$value = trim( ini_get( 'memory_limit' ) );

		switch ( strtolower( substr( $value, -1 ) ) )
		{
			case 'g':
				$value *= 1024;
			case 'm':
				$value *= 1024;
			case 'k':
				$value *= 1024;
		}

		return $value;
	}

}

// $image = '/Users/martin/Desktop/Himmelbett.JPG';
// $image = '/Users/martin/Desktop/testbild.jpg';
// $text = '© 2013 by Martin Wandelt';
// $watermark = '/Users/martin/Desktop/watermark.png';
// $font = 'Batang.ttf';
// $size = 16;
// $color = 'ffffff';
// $opacity = 70;
// $rotation = 45;
// $alignment = 13;
// $output_file_path = '/Users/martin/Desktop/output.jpg';
// 
// require_once 'metadata_manager.class.php';
// ini_set( 'memory_limit', '50M' );
// $ip = new Image_Processor( array( 'watermarkMargin' => 2 ) );
// if ( ! $ip->load( $image, TRUE ) )
// {
// 	die( 'Not enough memory.' );
// }

// var_dump( $ip->metadata_irb );
// var_dump( $ip->metadata_xmp );

// $ip->resize( 200 );
// $ip->add_watermark_text( $text, 11, $size, $font, $color, $opacity, $rotation );
// $ip->add_watermark_text( $text, 22, 0.7, $font, $color, $opacity, 0 );
// $ip->add_watermark_text( $text, $alignment, $size, $font, $color, $opacity, $rotation );
// $ip->add_watermark_image( $watermark, 11 );
// $ip->add_watermark_image( $watermark, 12 );
// $ip->add_watermark_image( $watermark, 13 );
// $ip->add_watermark_image( $watermark, 21 );
// $ip->add_watermark_image( $watermark, 0.5, 22, 30 );
// $ip->add_watermark_image( $watermark, 23 );
// $ip->add_watermark_image( $watermark, 31 );
// $ip->add_watermark_image( $watermark, 32 );
// $ip->add_watermark_image( $watermark, 33 );
// $ip->write( $output_file_path );
// 
// $peak = memory_get_peak_usage();
// list( $width, $height ) = getimagesize( $image );
// $size = $width * $height * 3;
// echo 'Dateigröße: ' . format_bytes( $size ) . "\n";
// echo 'Speicherverbrauch: ' . format_bytes( $peak ) . ' (das ' . round( $peak / $size, 1 ) . '-fache von ' . format_bytes( $size ) . ')';
// 
// 
// function format_bytes( $size )
//  {
//     $unit = array( 'B', 'kB', 'MB', 'GB', 'TB', 'PB' );
//     return round( $size / pow( 1024, ( $i = floor( log( $size, 1024 ) ) ) ), 2 ) . ' ' . $unit[ $i ];
//  }

// end of file
