<?php

// TODO: Kommentare entfernen / umschreiben


class Metadata_Manager {

	var $file;
	var $header_data = NULL;
	var $irb_data = NULL;
	var $iptc_data = NULL;
	var $xmp_data = NULL;

	
	function Metadata_Manager( $file )
	{
		$this->file = $file;
		$this->header_data = $this->get_header_data();
	}


	function set_iptc( $data )
	{
		// Example: $data = array( '2:25' => 'foo', '2:105' => 'bar' )

		$iptc = $this->get_iptc_data();
		$count = count( $iptc );

		// alte Daten entfernen

		for ( $i = 0; $i < $count; $i++ )
		{
			if ( isset( $data[ $iptc[ $i ]['IPTC_Type'] ] ) )
			{
				unset( $iptc[ $i ] );
			}
		}

		$iptc = array_values( $iptc ); // Neuindizierung erzwingen

		// neue Daten hinzufügen

		foreach ( $data as $key => $value )
		{
			foreach ( (array) $value as $item )
			{
				$iptc[] = array(
					'IPTC_Type' => $key,
					'RecData' => $item 
				);
			}
		}

		$this->put_iptc_data( $iptc );
		return $iptc;
	}


	function write( $outputfile )
	{
		$image_data = $this->get_image_data( $this->file );

		if ( $image_data === FALSE || $image_data === NULL )
		{
			return FALSE;
		}

		foreach ( $this->header_data as $segment )
		{
			if ( strlen( $segment['SegData'] ) > 0xfffd )
			{
				return FALSE; // Header is larger than the maximum size
			}
		}

		ignore_user_abort( TRUE );
		
		if ( gettype( $outputfile ) == 'resource' )
		{
			$handle = $outputfile;
		}
		else
		{
			$handle = fopen( $outputfile, 'wb' );
		}

		if ( ! $handle  )
		{
			return FALSE; // Could not open file for writing
		}

		fwrite( $handle, "\xFF\xD8" ); // Write SOI

		foreach ( $this->header_data as $segment )
		{
			fwrite( $handle, sprintf( "\xFF%c", $segment['SegType'] ) ); // Write segment marker
			fwrite( $handle, pack( 'n', strlen( $segment['SegData'] ) + 2 ) ); // Write segment size
			fwrite( $handle, $segment['SegData'] ); // Write segment data
		}

		fwrite( $handle, $image_data ); // Write image data
		fwrite( $handle, "\xFF\xD9" ); // Write EOI

		if ( gettype( $outputfile ) == 'string' )
		{
			fclose( $handle );
		}

		ignore_user_abort( FALSE );
		return TRUE;
	}


	function deliver( $filename = '' )
	{
		$handle = tmpfile();

		if ( ! $handle  )
		{
			die( 'Could not open temporary file for writing' );
		}

		if ( ! $this->write( $handle ) )
		{
			die( 'Could not create image file' );
		}

		rewind( $handle );
		$stat = fstat( $handle );
		header( 'Content-Description: File Transfer' );
		header( 'Content-Type: image/jpeg' );
		header( 'Content-Transfer-Encoding: binary' );
		header( 'Expires: 0' );
		header( 'Cache-Control: must-revalidate, post-check=0, pre-check=0' );
		header( 'Pragma: public' );
		header( 'Content-Length: ' . $stat['size'] );

		if ( empty( $filename ) )
		{
			header( 'Content-Disposition: inline; filename="' . basename( $this->file ) . '"' );
		}
		else
		{
			header( 'Content-Disposition: attachment; filename="' . $filename . '"' );
		}

		ob_end_clean();

		while ( ! feof( $handle ) )
		{
			print( fread( $handle, 1024*8 ) );
			flush();
			if ( connection_status() != 0 )
			{
				fclose( $handle );
				exit;
			}
		}

		fclose( $handle );
		exit;
	}


	function get_iptc_data()
	{
		if ( $this->iptc_data !== NULL )
		{
			return $this->iptc_data;
		}

		$irb = $this->get_irb_data();
		$count = count( $irb );

		for( $i = 0; $i < $count; $i++ )
		{
			if ( $irb[ $i ]['ResID'] == 0x0404 )
			{
				$data = $irb[ $i ]['ResData'];
				break;
			}
		}

		if ( empty( $data ) )
		{
			return FALSE;
		}

		$pos = 0;
		$length = strlen( $data );
		$this->iptc_data = array( );

		while( $pos < $length )
		{
			if ( strlen( substr( $data, $pos ) ) < 5 )
			{
				// Not enough data left for a record - Probably corrupt data
				// However, return partial data
				return $this->iptc_data;
			}

			// Unpack data from IPTC record:
			// First byte - IPTC Tag Marker - always 28
			// Second byte - IPTC Record Number
			// Third byte - IPTC Dataset Number
			// Fourth and fifth bytes - two byte size value
			$iptc_raw = unpack( 'CIPTC_Tag_Marker/CIPTC_Record_No/CIPTC_Dataset_No/nIPTC_Size', substr( $data, $pos ) );

			// Skip position over the unpacked data
			$pos += 5;

			// Construct the IPTC type string
			$iptctype = sprintf( '%01d:%02d', $iptc_raw['IPTC_Record_No'], $iptc_raw['IPTC_Dataset_No'] );

			// Check if there is sufficient data for reading the record contents
			if ( strlen( substr( $data, $pos, $iptc_raw['IPTC_Size'] ) ) !== $iptc_raw['IPTC_Size'] )
			{
				// Not enough data left for record content - Probably corrupt data
				// However, return partial data
				return $this->iptc_data;
			}

			// Add the IPTC record to the output array
			$this->iptc_data[] = array( 
				'IPTC_Type' => $iptctype ,
				'RecData' => substr( $data, $pos, $iptc_raw['IPTC_Size'] ) 
			);

			// Skip over the IPTC record data
			$pos += $iptc_raw['IPTC_Size'];
		}

		return $this->iptc_data;
	}


	function put_iptc_data( $new_iptc_block )
	{
		$this->iptc_data = $new_iptc_block;
		$irb_data = $this->get_irb_data();
		$iptc_block_pos = count( $irb_data );

		for( $i = 0; $i < $iptc_block_pos; $i++ )
		{
			if ( $irb_data[ $i ]['ResID'] == 0x0404 )
			{
				$iptc_block_pos = $i;
			}
		}

		$iptc_data = '';

		foreach ( $new_iptc_block as $record )
		{
			$type = sscanf( $record['IPTC_Type'], '%d:%d' );

			// Write the IPTC-NAA IIM Tag Marker, Record Number, Dataset Number and Data Size to the packed output data string
			$iptc_data .= pack( 'CCCn', 28, $type[0], $type[1], strlen( $record['RecData'] ) );

			// Write the IPTC-NAA IIM Data to the packed output data string
			$iptc_data .= $record['RecData'];
		}

		$irb_data[ $iptc_block_pos ] = array(
			'ResID' => 0x0404,
			'ResEmbeddedName' => "\x00\x00",
			'ResData' => $iptc_data 
		);

		$this->put_irb_data( $irb_data );
	}
	
	
	function get_xmp_data()
	{
		if ( $this->xmp_data !== NULL )
		{
			return $this->xmp_data;
		}

		$count = count( $this->header_data );

		for ( $i = 0; $i < $count; $i++ )
		{
			if ( $this->header_data[ $i ]['SegType'] == 0xE1 )
			{
				if ( substr( $this->header_data[ $i ]['SegData'], 0, 29 ) == "http://ns.adobe.com/xap/1.0/\x00" )
				{
					$this->xmp_data = substr( $this->header_data[ $i ]['SegData'], 29 );
					return $this->xmp_data;
				}
			}
		}

		$this->xmp_data = '';
		return '';
	}
	
	
	function put_xmp_data( $xmp_string )
	{
		for ( $i = 0; $i < count( $this->header_data ); $i++ )
		{
			if ( $this->header_data[ $i ]['SegType'] == 0xE1 
					&& substr( $this->header_data[ $i ]['SegData'], 0, 29 ) == "http://ns.adobe.com/xap/1.0/\x00" )
			{
				// Found a preexisting XMP/RDF block - Replace it with the new one and return.
				$this->header_data[ $i ]['SegData'] = "http://ns.adobe.com/xap/1.0/\x00" . $xmp_string;
				return;
			}
		}

		$i = 0;

		// Loop until a block is found that isn't an APP0 or APP1
		while ( $this->header_data[ $i ]['SegType'] == 0xE0 || $this->header_data[$i]['SegType'] == 0xE1 )
		{
			$i++;
		}

		// Insert a new XMP/RDF APP1 segment at the specified point.
		array_splice( $this->header_data, $i, 0, array( array(
			'SegType' => 0xE1,
			'SegData' => "http://ns.adobe.com/xap/1.0/\x00" . $xmp_string ) ) 
		);
	}


	function get_irb_data()
	{
		if ( $this->irb_data !== NULL )
		{
			return $this->irb_data;
		}

		// Photoshop Image Resource blocks can span several JPEG APP13 segments, 
		// so we need to join them up if there are more than one

		$irb = '';
		$count = count( $this->header_data );

		for ( $i = 0; $i < $count; $i++ )
		{
			if ( $this->header_data[ $i ]['SegType'] == 0xED 
					&& substr( $this->header_data[ $i ]['SegData'], 0, 14) == "Photoshop 3.0\x00" )
			{
				$irb .= substr ( $this->header_data[ $i ]['SegData'], 14 );
			}
		}

		if ( $irb == '' )
		{
			$this->irb_data = array();
			return FALSE;
		}

		$length = strlen( $irb );
		$pos = strpos( $irb, '8BIM' );

		while ( $pos !== FALSE )
		{
			// Skip the position over the 8BIM characters
			$pos += 4;

			// Next two characters are the record ID - denoting what type of record it is.
			$id = ord( $irb[ $pos ] ) * 256 + ord( $irb[ $pos + 1 ] );

			// Skip the position over the two record ID characters
			$pos += 2;

			// Next comes a Record Name - usually not used, but it should be a null terminated string, padded with 0x00 to be an even length
			$name_pos = $pos;

			// NOTE: Photoshop does not process resource names according to the standard:
			// "Adobe Photoshop 6.0 File Formats Specification, Version 6.0, Release 2, November 2000"
			//
			// The resource name is actually formatted as follows:
			// One byte name length, followed by the null terminated ascii name string.
			// The field is then padded with a Null character if required, to ensure that the
			// total length of the name length and name is even.

			// Name - process it
			// Get the length
			$name_length = ord ( $irb[ $name_pos ] );

			// Total length of name and length info must be even, hence name length must be odd
			// Check if the name length is even,
			if ( $name_length % 2 == 0 )
			{
				// add one to length to make it odd
				$name_length ++;
			}

			$pos += $name_length + 1;

			// Next is a four byte size field indicating the size in bytes of the record's data  - MSB first
			$datasize = ord( $irb[ $pos ] ) * 16777216 + ord( $irb[ $pos + 1 ] ) * 65536 + ord( $irb[ $pos + 2 ] ) * 256 + ord( $irb[ $pos + 3 ] );
			$pos += 4;

			// Get the description for this resource
			// Check if this is a Path information Resource, since they have a range of ID's
			if ( ( $id >= 0x07D0 ) && ( $id <= 0x0BB6 ) )
			{
				$description = "ID Info : Path Information (saved paths).";
			}
			else
			{
				$description = isset( $this->photoshop_irb_descriptions[ $id ] ) ? $this->photoshop_irb_descriptions[ $id ] : '';
			}

			$this->irb_data[] = array(
				"ResID" => $id,
				"ResEmbeddedName" => trim( substr ( $irb, $name_pos + 1,  $name_length ) ),
				"ResData" => substr( $irb, $pos, $datasize ) 
			);

			// Take into account that the record is stored padded with 0x00 characters to make the size even
			$pos += $datasize + ( $datasize % 2 );
			
			if ( $pos >= $length )
			{
				break;
			}

			$pos = strpos( $irb, '8BIM', $pos );
		}

		return $this->irb_data;
	}
	
	
	function put_irb_data( $new_data )
	{
		// Delete all existing Photoshop IRB blocks - the new ones will replace them

		$count = count( $this->header_data );

		for ( $i = 0; $i < $count ; $i++ )
		{
			if ( $this->header_data[ $i ]['SegType'] == 0xED
					&& substr( $this->header_data[ $i ]['SegData'], 0, 14 ) == "Photoshop 3.0\x00" )
			{
				unset( $this->header_data[ $i ] );
			}
		}

		// Retrieve the Packed Photoshop IRB Data
		// Change: Moved code into pack_Photoshop_IRB_Data to allow TIFF writing as of 1.11

		$packed_data = $this->_pack_irb_data( $new_data );

		// Cycle through the header segments in reverse order to find where to put the APP13 block (after any APP0 to APP12 blocks)

		$i = count( $this->header_data ) - 1;

		while ( $i >= 0 && ( $this->header_data[ $i ]['SegType'] > 0xED || $this->header_data[ $i ]['SegType'] < 0xE0 ) )
		{
			$i--;
		}

		// Cycle through the packed output data until it's size is less than 32000 bytes, outputting each 32000 byte block to an APP13 segment

		while ( strlen( $packed_data ) > 32000 )
		{
			// Write a 32000 byte APP13 segment

			array_splice( $this->header_data, $i + 1, 0, array(
				'SegType' => 0xED,
				'SegData' => "Photoshop 3.0\x00" . substr( $packed_data, 0, 32000 ) ) 
			);

			// Delete the 32000 bytes from the packed output data, that were just output
			$packed_data = substr_replace( $packed_data, '', 0, 32000 );
			$i++;
		}

		// Write the last block of packed output data to an APP13 segment - Note array_splice doesn't work with multidimensional arrays, hence inserting a blank string
		array_splice( $this->header_data, $i + 1 , 0, '' );

		$this->header_data[ $i + 1 ] =  array(
			'SegType' => 0xED,
			'SegData' => "Photoshop 3.0\x00" . $packed_data 
		);
	}
	
	
	function _pack_irb_data( $data )
	{
		$result = '';

		foreach ( $data as $resource )
		{
			if ( strlen( $resource['ResData'] ) == 0 )
			{
				continue;
			}

			$result .= pack( 'a4n', '8BIM', $resource['ResID'] );

			// NOTE: Photoshop does not process resource names according to the standard :
			// "Adobe Photoshop 6.0 File Formats Specification, Version 6.0, Release 2, November 2000"
			//
			// The resource name is actually formatted as follows:
			// One byte name length, followed by the null terminated ascii name string.
			// The field is then padded with a Null character if required, to ensure that the
			// total length of the name length and name is even.

			// Append Name Size
			$result .= pack( 'c', strlen( trim( $resource['ResEmbeddedName'] ) ) );

			// Append the Resource Name to the packed output data
			$result .= trim( $resource['ResEmbeddedName'] );

			// If the resource name is even length, then with the addition of
			// the size it becomes odd and needs to be padded to an even number
			if ( strlen( trim( $resource['ResEmbeddedName'])  ) % 2 == 0 )
			{
				// then it needs to be evened up by appending another null
				$result .= "\x00";
			}

			// Append the resource data size to the packed output data
			$result .= pack( 'N', strlen( $resource['ResData'] ) );

			// Append the resource data to the packed output data
			$result .= $resource['ResData'];

			// If the resource data is odd length,
			if ( strlen( $resource['ResData'] ) % 2 == 1 )
			{
				// then it needs to be evened up by appending another null
				$result .= "\x00";
			}
		}

		return $result;
	}

	
	function get_header_data()
	{
		$handle = fopen( $this->file, 'rb' );

		if ( ! $handle  )
		{
			return FALSE;
		}

		if ( fread( $handle, 2 ) != "\xFF\xD8" )
		{
			fclose( $handle );
			return FALSE; // No SOI (FF D8) at start of file - This probably isn't a JPEG file
		}

		$result = array();
		$data = fread( $handle, 2 );

		while ( $data[0] == "\xFF" && $data[1] != "\xD9" && ! feof( $handle ) )
		{
			// Check that the segment marker is not a Restart marker - restart markers don't have size or data after them
			if (  ( ord( $data[1] ) < 0xD0 ) || ( ord( $data[1] ) > 0xD7 ) )
			{
				// Segment isn't a Restart marker
				// Read the next two bytes (size)
				$sizestr = fread( $handle, 2 );

				// convert the size bytes to an integer
				$decodedsize = unpack( 'nsize', $sizestr );

				// Save the start position of the data
				$segdatastart = ftell( $handle );

				// Read the segment data with length indicated by the previously read size
				$segdata = fread( $handle, $decodedsize['size'] - 2 );

				// Store the segment information in the output array
				$result[] = array(
					'SegType' => ord( $data[1] ),
					'SegDataStart' => $segdatastart,
					'SegData' => $segdata
				);
			}

			if ( $data[1] == "\xDA" )
			{
				// This is a SOS (Start Of Scan) segment, thus there is no more header data
				break;
			}

			$data = fread( $handle, 2 );
		}

		fclose( $handle );
		return $result;
	}


	function get_image_data()
	{
		$handle = fopen( $this->file, 'rb' );

		if ( ! $handle  )
		{
			return FALSE;
		}

		$data = fread( $handle, 2 );

		if ( $data != "\xFF\xD8" )
		{
			fclose( $handle );
			return FALSE; // No SOI at start of file
		}

		$data = fread( $handle, 2 );

		while ( $data[0] == "\xFF" && $data[1] != "\xD9" && ! feof( $handle ) )
		{
			if ( ord( $data[1] ) < 0xD0 || ord( $data[1] ) > 0xD7 )
			{
				// Segment isn't a Restart marker (does not have size or data after them)
				// Read the next two bytes (size)
				$sizestr = fread( $handle, 2 );

				// convert the size bytes to an integer
				$decodedsize = unpack( 'nsize', $sizestr );

				// Read the segment data with length indicated by the previously read size
				$segdata = fread( $handle, $decodedsize['size'] - 2 );
			}

			if ( $data[1] == "\xDA" )
			{
				// This is a SOS (Start Of Scan) segment, thus there is no more header data - the compressed image data follows

				$compressed_data = fread( $handle, filesize( $this->file ) );
				$compressed_data = substr( $compressed_data, 0, strpos( $compressed_data, "\xFF\xD9" ) ); // Strip off EOI and anything after
				fclose( $handle );
				return $compressed_data;
			}

			$data = fread( $handle, 2 );
		}

		fclose( $handle );
		return FALSE;
	}
}


// end of file